//
// Created by michal on 25.11.18.
//

#include <soc/dport_reg.h>
#include <freertos/FreeRTOS.h>
#include <freertos/portmacro.h>

#include "spi.h"


static bool dma_acquired[2] = { false, false };
static portMUX_TYPE dma_reg_spinlock = portMUX_INITIALIZER_UNLOCKED;

esp_err_t SPI_DMA_Channel_Try_Acquire(SPI_Device_t device)
{
    assert(SPI_DEVICE_CHECK(device));

    vPortCPUAcquireMutex(&dma_reg_spinlock);

    esp_err_t ret = ESP_FAIL;

    if (!dma_acquired[0])
    {
        dma_acquired[0] = true;
        DPORT_SET_PERI_REG_BITS(DPORT_SPI_DMA_CHAN_SEL_REG, 3, 1, device * 2 - 2);
        ret = ESP_OK;
    } else if (!dma_acquired[1])
    {
        dma_acquired[1] = true;
        DPORT_SET_PERI_REG_BITS(DPORT_SPI_DMA_CHAN_SEL_REG, 3, 2, device * 2 - 2);
        ret = ESP_OK;
    }

    vPortCPUReleaseMutex(&dma_reg_spinlock);

    return ret;
}

void SPI_DMA_Channel_Free(SPI_Device_t device)
{
    assert(SPI_DEVICE_CHECK(device));

    vPortCPUAcquireMutex(&dma_reg_spinlock);

    uint8_t channel = DPORT_GET_PERI_REG_BITS2(DPORT_SPI_DMA_CHAN_SEL_REG, 3, device * 2 - 2);
    DPORT_SET_PERI_REG_BITS(DPORT_SPI_DMA_CHAN_SEL_REG, 3, 0, device * 2 - 2);

    assert(channel == 1 || channel == 2);
    dma_acquired[channel - 1] = false;

    vPortCPUReleaseMutex(&dma_reg_spinlock);
}
